<?php

namespace app\models\entity;

/**
 * This is the model class for table "accounts".
 *
 * @property int $id
 * @property string $name
 * @property int $cmdlevel
 * @property string $phone
 * @property string $password
 * @property string $login
 * @property string $email
 * @property string $skype
 * @property int $company_id
 * @property int $active_company
 * @property string $active_device_id
 * @property string $active_task_ids
 * @property string $created
 * @property string $jobtypes
 * @property string $last_seen_mobile
 * @property string $last_seen_web
 *
 * @property Companies $company
 * @property AccountsJobtypes[] $accountsJobtypes
 * @property string $uuid [char(36)]
 * @property string $last_push_message_sent [datetime]
 * @property string $employee_status [varchar(255)]
 * @property string $position [char(36)]
 * @property string $repair_site [char(36)]
 * @property string $hash [varchar(255)]
 * @property bool $basic_jobs_count [tinyint(3) unsigned]
 * @property int $shift_id [smallint(5)]
 * @property string $android_token_id [varchar(255)]
 * @property bool $overplan_mode [tinyint(3) unsigned]
 * @property string $last_action [datetime]
 * @property bool $quit_set [tinyint(4)]
 *
 * @property FaceFeature $faceFeature
 */
class Accounts extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'accounts_internal';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'cmdlevel', 'password', 'login', 'active_company',  'jobtypes'], 'required'],
            [['cmdlevel', 'company_id', 'active_company'], 'integer'],
            [['created', 'last_seen_mobile', 'last_seen_web'], 'safe'],
            [['name', 'phone', 'password', 'login', 'email', 'active_device_id', 'jobtypes'], 'string', 'max' => 255],
            [['skype'], 'string', 'max' => 32],
            //[['active_task_ids'], 'string', 'max' => 100],
            //[['login'], 'unique'],
            //[['name'], 'unique'],
            //[['company_id'], 'exist', 'skipOnError' => true, 'targetClass' => Companies::className(), 'targetAttribute' => ['company_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'cmdlevel' => 'Cmdlevel',
            'phone' => 'Phone',
            'password' => 'Password',
            'login' => 'Login',
            'email' => 'Email',
            'skype' => 'Skype',
            'company_id' => 'Company ID',
            'active_company' => 'Active Company',
            'active_device_id' => 'Active Device ID',
            'active_task_ids' => 'Active Task Ids',
            'created' => 'Created',
            'jobtypes' => 'Jobtypes',
            'last_seen_mobile' => 'Last Seen Mobile',
            'last_seen_web' => 'Last Seen Web',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCompanies()
    {
        return $this->hasOne(Companies::className(), ['id' => 'company']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountsJobtypes()
    {
        return $this->hasMany(AccountsJobtypes::className(), ['account_id' => 'id'])->orderBy(['priority' => SORT_ASC]);
    }
    
    public function getAccountsJobtypesPriority()
    {
        return $this->hasMany(AccountsJobtypes::className(), ['account_id' => 'id'])->where(['priority' => 0]);
    }
    
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getJobtype()
    {
        return $this->hasMany(Jobtypes::class, ['id' => 'jobtype_id'])
            ->via('accountsJobtypes');
    }
    
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTasks()
    {
        return $this->hasMany(Tasks::class, ['assignees_arr' => 'id'])->andWhere(['<>','status', 5]);
    }

    public function isAdmin()
    {
        return $this->cmdlevel == 10;
    }

    public function getFaceFeature()
    {
        return $this->hasOne(FaceFeature::className(), ['id_account' => 'id']);
    }
    
    public function getFaceFeatureCurrent($id = null)
    {
        $row_last = (new \yii\db\Query())
            ->select(['id_account','face_feature','date'])
            ->from('face_feature')
            ->where(['id_account' => $id])
            ->orderBy(['id' => SORT_DESC])
            ->one();
        
        return $row_last;
    }

    /**
     * Возвращает массив ID должностей
     *
     * @return int[]
     */
    public function getPositionIds()
    {
        $positions = AccountsJobtypes::find()
            ->select('jobtype_id')
            ->where(['account_id' => $this->id])
            ->asArray()
            ->all()
        ;

        return array_column($positions, 'jobtype_id');
    }

    /**
     * Возвращает информацию по должностям
     *
     * @return array
     */
    public function getPositionsList()
    {
        $positionIds = $this->getPositionIds();
        $accountJobTypes = AccountsJobtypes::find()->all();
        $priorities = [];
        foreach ($accountJobTypes as $type) {
            $priorities[$type->jobtype_id] = $type->priority;
        }
        $positions = [];
        foreach (Jobtypes::findAll($positionIds) as $position) {
            $positions[] = [
                'id' => $position->id,
                'name' => $position->name,
                'main' => $priorities[$position->id],
            ];
        }

        return $positions;
    }

    /**
     * Возвращает названия должностей
     *
     * @return string[]
     */
    public function getPositionsNames()
    {
        $positionIds = $this->getPositionIds();
        $positions = [];
        foreach (Jobtypes::findAll($positionIds) as $position) {
            $positions[] = $position->name;
        }

        return $positions;
    }

    /**
     * Проверяет онлайн ли пользователь в мобильном приложении
     *
     * @param $time
     * @return bool
     * @throws \Exception
     */
    public function isOnline(): bool
    {
        if (null == $this->last_seen_mobile) return false;

        $now = new \DateTime();
        $tenMin = new \DateInterval('PT10M');
        $now->sub($tenMin);

        return $now->format('Y-m-d H:i:s') <= $this->last_seen_mobile;
    }
}
