<?php /** Created by Anton on 11.02.2020. */

namespace app\models\entity\ar;

use yii\db\ActiveRecord;

/**
 * This is the model class for table "ar_stages".
 *
 * @property int $id
 * @property int $scenario_id [int(10)]
 * @property int $object_id [int(10)]
 * @property string $message
 * @property int $priority [int(10)]
 * @property-read TargetObject $object
 * @property-read Media[] $medias
 * @property-read Scenario $scenarios
 */
class Stage extends ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'ar_stages';
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'scenario_id' => 'ID сценария',
            'object_id' => 'ID объекта',
            'message' => 'Сообщение',
            'priority' => 'Приоритет'
        ];
    }

    public function getObject()
    {
        return $this->hasOne(TargetObject::class, ['id' => 'object_id']);
    }

    public function getMedias()
    {
        return $this->hasMany(Media::class, ['stage_id' => 'id']);
    }

    public function getScenarios()
    {
        return $this->hasOne(Scenario::class, ['id' => 'scenario_id']);
    }

    public function fillFromArray(array $params)
    {
        foreach ($params as $key => $fieldName) {
            $this->$key = $params[$key];
        }
    }

    public function asArray($withMedias = false)
    {
        $stage = [];
        foreach ($this->fields() as $fieldName) {
            $stage[$fieldName] = $this->$fieldName;
        }

        if ($withMedias) {
            $medias = [];
            /** @var Media $media */
            foreach ($this->medias as $media) {
                $medias[] = $media->asArray();
            }
            $stage['medias'] = $medias;
        }

        return $stage;
    }
}
