<?php

namespace app\models\procedure;

use Yii;
use yii\base\Model;
use app\models\entity\Tasks;
use app\models\entity\Accounts;
use app\models\entity\Tasktypes;
use app\models\entity\ProjectsLocotech;

class AddTaskIntoProjectModel extends Model 
{
    private $taskId;
    private $tmptype;
    private $tmptask;
    private $tasktype;
    private $companyId;
    private $projectId;
    private $initiator;
    private $taskMainId;
    private $taskGuid = '';
    private $listTasktypes;

    public function execute($initiator, $projectId, $tasktype)
    {
    
        $this->initiator = $initiator;
        $this->tasktype = $tasktype;
        $this->projectId = $projectId;
        //  Проверяем возможность добавления задачи указанного типа в указанный проект.
        
        // 1 Для начала выбираем ID компании, в которую входит инициатор добавления задачи. У инициатора cmdlevel > 1.
        if( !$this->getCompanyId() ) {
            return false;
        }
        
        // 2 Затем проверяем, есть ли затребованный тип задач в указанной компании.
        
        if( !$this->checkTasktype() ) {
            return false;
        }
 
        // 3 Затем проверяем, существует ли затребованный проект в указанной компании.
        
        if( !$this->isIssetProject() ) {
            return false;
        }
        
        // 4 Метка в поле text для созданной главной задачи.
        
        if( !$this->setLabel() ) {
            return false;
        }
        
        // 5 Добавляем задачу указанного типа.
        
        $this->addTask();
        
        // 6 Запоминаем ID главной задачи.
        
        if( !$this->getMainTaskId() ) {
            return false;
        }
        
        // 7 Для каждого типа задач добавляем новую задачу и привязываем её к родительской.
        
        $this->getlistTasktypes();
        $this->setParentTask();
        
    }
    
    
    
        // 1 Для начала выбираем ID компании, в которую входит инициатор добавления задачи. У инициатора cmdlevel > 1.
        
        private function getCompanyId()
        {
            $accountsEntity = Accounts::findOne(['id' => $this->initiator, 'cmdlevel' > 1]);
            $this->companyId = $accountsEntity->company;
            if( null == $this->companyId ) {
                return false;
            }
            return true;
        }
        
        // 2 Затем проверяем, есть ли затребованный тип задач в указанной компании.
        
        private function checkTasktype()
        {
            $tasktypesEntity = Tasktypes::findOne(['id' => $this->tasktype , 'company' = $this->companyId,'main_task' => 1]);
            $this->taskId = $tasktypesEntity->id;
            if( null == $this->taskId ) {
                return false;
            }
            return true;
        }
 
        // 3 Затем проверяем, существует ли затребованный проект в указанной компании.
        
        private function isIssetProject()
        {
            $projectsLocotechEntity = ProjectsLocotech::findOne(['id' => $this->projectId, 'company' => $this->companyId]);
            $this->projectId = $projectsLocotechEntity->id;
            if (null == $this->projectId) {
                return false;
            }
            return true;
        }
        
        // 4 Метка в поле text для созданной главной задачи.
        
        private function setLabel($uuid = false)
        {
            $query = 'select UUID()';
            $uuid = Yii::$app->db->createCommand('select UUID() as uuid')->queryOne();
            if ( $uuid ) {
                return $uuid["uuid"];
            }
            $this->taskGuid = $uuid["uuid"];
            if ( null == $this->taskGuid){
                return false;
            } 
            return true;
        }
        
        // 5 Добавляем задачу указанного типа.
        
        private function addTask()
        {
            $tasksEntity = new Tasks();
            $tasksEntity->status        = 1; 
            $tasksEntity->parent_id     = 0;
            $tasksEntity->type          = $this->tasktype;
            $tasksEntity->input_id      = $this->projectId;
            $tasksEntity->text          = $this->taskGuid;
            $tasksEntity->project_id    = 0;
            $tasksEntity->assignees_arr = 0;
            $tasksEntity->priority      = 1;
            $tasksEntity->assignedby    = 0;
            $tasksEntity->created       = date('Y-m-d H:i:S');
            $tasksEntity->save();
            
        }
        
        // 6 Запоминаем ID главной задачи.
        
        private function getMainTaskId()
        {
            $tasksEntity = Tasks::find()->where(['like','text', $this->taskGuid])->one();
            $this->taskMainId = $tasksEntity->id;
            if ( null == $this->taskMainId ){
                return false;
            } 
            return true;
        }
        
        // 7 Для каждого типа задач добавляем новую задачу и привязываем её к родительской.
        
        private function getlistTasktypes()
        {
            $TasktypesEntity = Tasktypes::find()
                                    ->leftJoin('tasktypes as ttp', 'instr(ttp.subtasks, tasktypes.id) > 0')
                                    ->select(['tasktypes.id'])
                                    ->where(['ttp.id' => $this->tasktype])
                                    ->all();
            $this->listTasktypes = $TasktypesEntity;
            
        }
        
        private function setParentTask()
        {
            foreach ( $this->listTasktypes as $tasktype) {
                    $uuid = $this->setLabel(true);
                    $tasksEntity = new Tasks();
                    $tasksEntity->status        = 1
                    $tasksEntity->parent_id     = $this->taskMainId;
                    $tasksEntity->type          = $tasktype->id;
                    $tasksEntity->input_id      = $this->projectId;
                    $tasksEntity->text          = $uuid;
                    $tasksEntity->project_id    = 0;
                    $tasksEntity->assignees_arr = 0;
                    $tasksEntity->priority      = 0;
                    $tasksEntity->assignedby    = 0;
                    $tasksEntity->created       = date('Y-m-d H:i:S');
                    $tasksEntity->save();
                    
                    $projectsLocotechEntity = ProjectsLocotech::findOne($this->projectId);
                    $projectsLocotechEntity->tasks = $tasktype->id . ',' . $tasksEntity->id;
                    $projectsLocotechEntity->save();
            }
        }
}

/*
add_task_whole_pack: BEGIN
	declare taskId integer;
    declare tmptype integer;
    declare tmptask integer;
    declare companyId integer;
    declare projectId integer;
    declare taskGuid varchar(255);
	declare done integer;
	declare cur cursor for select ttx.id from tasktypes ttx left join tasktypes ttp on instr(ttp.subtasks, ttx.id) > 0 where ttp.id = tt;
    declare continue handler for not found set done = true;
    set done = false;

    #Проверяем возможность добавления задачи указанного типа в указанный проект.
    #Для начала выбираем ID компании, в которую входит инициатор добавления задачи. У инициатора cmdlevel > 1.
    set companyId = (select company from accounts where id = initiator and cmdlevel > 1);
    if (companyId is null) then leave add_task_whole_pack;
    end if;

    #Затем проверяем, есть ли затребованный тип задач в указанной компании.
    set taskId = (select id from tasktypes where id = tt and company = companyId and main_task = 1);
    if (taskId is null) then leave add_task_whole_pack;
    end if;

    #Затем проверяем, существует ли затребованный проект в указанной компании.
    set projectId = (select id from projects_locotech where id = project and company = companyId);
    if (projectId is null) then leave add_task_whole_pack;
    end if;

    #Метка в поле text для созданной главной задачи.
    set taskGuid = (select UUID());
    
    insert into text_log (msg) values (taskGuid);

	#Добавляем задачу указанного типа.
	insert into tasks
		(status, parent_id, type, input_id, text, project_id, assignees_arr, priority, assignedby, created)
		values (1, 0, tt, projectId, taskGuid, 0, 0, 1, 0, NOW());

	#Запоминаем ID главной задачи.
	set taskId = (select id from tasks where text like taskGuid);

	#Для каждого типа задач добавляем новую задачу и привязываем её к родительской.
	open cur;
    task_loop: loop
		fetch cur into tmptype;
        if done then leave task_loop;
        end if;
		set taskGuid = (select UUID());
		insert into tasks
			(status, parent_id, type, input_id, text, project_id, assignees_arr, priority, assignedby, created)
			values (1, taskId, tmptype, projectId, taskGuid, 0, 0, 0, 0, NOW());
		set tmptask = (select id from tasks where text like taskGuid);
		update projects_locotech
			set tasks = concat(tasks, ',', cast(tmptask as char))
            where id = projectId;
    end loop;
	close cur;
END
*/
