<?php

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Collections\Criteria;
/**
 * @Entity @Table(name="locomotive_series")
 */

class LocoSeries extends BaseDBO implements ArrayExpressible
{
    /**
     * @Column(type="integer") @GeneratedValue
     */
    protected $id;

    /**
     * @Id
     * @var string
     * @Column(type="string")
     */
    public $name;

    /**
     * One section has many projects. This is the inverse side.
     * @OneToMany(targetEntity="Section", mappedBy="series", cascade={"persist"}, orphanRemoval = true)
     *  @OrderBy({"section_number" = "ASC"})
     */
    public $sections;

    /**
     * @ManyToMany(targetEntity="ProjectType",inversedBy="series")
     * @JoinTable(name="projecttypes_locoseries",
     *      joinColumns={@JoinColumn(name="locoseries", referencedColumnName="name")},
     *      inverseJoinColumns={@JoinColumn(name="projecttype_id", referencedColumnName="id")}
     * )
     */
    public $projecttypes;

    public function toArray() {
        return get_object_vars($this);
    }

    public function addSection($section, $save = false)
    {
        $this->sections[] = $section;
        $section->series = $this;
        if ($save)
            $this->Save();
    }

    public function projectsAmount()
    {
        $numproj = 0;
        foreach ($this->sections as $s)
        {
            $numproj += sizeof($s->projects);
        }
        return $numproj;
    }

    public function getSectionsWithUUID($asArray = false)
    {
        $expressionBuilder = Criteria::expr();
        $expression = $expressionBuilder->andX(
            $expressionBuilder->neq('uuid', null),
            $expressionBuilder->neq('locomotive_series',  "'$this->name'")
        );

        if (!$asArray)
            return $this->sections->matching(new Criteria($expression));
        else
        {
            $secarr = $this->sections->matching(new Criteria($expression));
            $ret = array();
            foreach ($secarr as $section)
            {
                array_push($ret, $section->toArray());
            }
            return $ret;
        }
    }

    public static function FindByName($loco_type)
    {
        global $entityManager;
        $productRepository = $entityManager->getRepository('LocoSeries');
        return $productRepository->findBy(array('name'=>$loco_type))[0];
    }

    public static function GetAll()
    {
        global $entityManager;
        $productRepository = $entityManager->getRepository('LocoSeries');
        return $productRepository->findAll();
    }

    public static function GetAllNames()
    {
        global $entityManager;
        $arrnames = array();
        $productRepository = $entityManager->getRepository('LocoSeries');
        $series = $productRepository->findAll();
        foreach ($series as $s)
        {
            $arrnames[] = $s->name;
        }
        sort($arrnames, SORT_STRING);
        return $arrnames;
    }
}