<?php

/**
 * @Entity @Table(name="sections")
 */
class Section extends BaseDBO implements ArrayExpressible
{
    /**
     * One section has many projects. This is the inverse side.
     * @OneToMany(targetEntity="Project", mappedBy="section")
     */
    public $projects;

     /**
     * @Id @Column(type="integer") @GeneratedValue
     */
    protected $id;

    /**
     * @ManyToOne(targetEntity="LocoSeries", inversedBy="sections")
     * @JoinColumn(name="locomotive_series", referencedColumnName="name")
     */
    public $series;

    /**
     * @var string
     *
     * @Column(type="string")
     */
    public $locomotive_series;

    /**
     * @var string
     *
     * @Column(type="integer")
     */
    private $section_number;

    /**
     * @var string
     *
     * @Column(type="string")
     */
    public $letter;

    /**
     * @var string
     *
     * @Column(type="string")
     */
    private $section_subnumber;

    /**
     * @var string
     *
     * @Column(type="string")
     */
    public $uuid;

    /**
     * One section has many projects. This is the inverse side.
     * @OneToMany(targetEntity="DiagnosticMap", mappedBy="section", cascade={"persist"})
     */
    public $diagnostic_maps;

    public function UpdateDiagnosticMap($save = false)
    {
        $subnum = $this->getSectionSubnumber();
        $params['subnum_section'] = $subnum;

        $time = date('Y-m-d\TH:i:s');
//        $params['placeOfRepair'] = "a84d0a15-08ec-11e5-829d-00155d6cab05";      //Братское
        $params['mes_time'] = $time;//"2019-10-28T13:00:00";
        //
//        echo "<br><b>Секция $subnum, ответ TWX:</b> ";
        $locomotiveModel = new app\models\LocomotiveModel();
        $result = $locomotiveModel->getSectionInfo($params);
        $json = json_decode($result);
        if ($json != null && $json->diagnosticMap != null) {
            $uuid = $json->diagnosticMap;
            $this->uuid = $json->section_id;
            $existed = DiagnosticMap::Find($uuid);
            if ($existed == null) {
                $dm = new DiagnosticMap();
                $dm->created = new DateTime("now");
                $dm->uuid = $uuid;
                $this->addDiagnosticMap($dm);
            }
            $this->diagnostic_map = $uuid;
        }

        if ($save)
            $this->Save();
        return $result;
    }


    public function addDiagnosticMap(DiagnosticMap $dm, $save = false) {
        $this->diagnostic_maps[] = $dm;
        $dm->section = $this;
        if ($save)
            $this->Save();
    }

    /**
     * @var string
     *
     * @Column(type="string")
     */
    public $diagnostic_map;

    public $projectsAmount;

    public function toArray() {
        return get_object_vars($this);
    }

    public function AddRemark($account, $text)
    {
            $diagnosticMap = $this->diagnostic_map;
            $accountUuid = $account->uuid;
//            $accountPosition = $account->position;
            if (!$diagnosticMap)
                $diagnosticMap = '25c46291-9eb1-11e8-80cf-005056011052';

            $locomotiveModel = new app\models\LocomotiveModel();
            $locomotiveModel->project_id = $this->id;
            $params['number'] = $this->getSectionNumber().$this->letter;//'0130А';
            $params['subnumber'] = $this->getSectionSubnumber(); //'11374006';
            $params['diagnosticMap'] = $diagnosticMap;
            $params['date'] = time();
            $params['worker'] = $accountUuid;//'bbd19a7f-bf7d-11e7-80cd-0050560110fa';
            //"ID":"f767d64e-60b6-4ded-8b49-92fc48349a06","Name":"Приемка перед СО"
            //9fc09716-7b2e-4f38-90c3-475192df9e7a: Осмотр мастером (перед СО)
            $params['source'] = "f767d64e-60b6-4ded-8b49-92fc48349a06"; //$accountPosition; //Вепрев: d4f49c18-e75a-11e2-aaad-ac162db0b20c, Феофилактов: 27ff8b22-ec10-11e2-aa9d-ac162db0b20c
            $params['description'] = $text;
            $params['isSCWorksNeeded'] = '';
            $params['plant'] = '';
            $params['sectionId'] = $this->uuid;
            $result = $locomotiveModel->addRemark($params);
            return $result;
//            return ["result" => $result, "params" => $params];
    }

    public static function FindBySeries($loco_type, $asArray = false)
    {
        global $entityManager;
        $qb = $entityManager->createQueryBuilder();

        $qb->select(array('s')) // string 'u' is converted to array internally
        ->from('Section', 's')
            ->where(
                $qb->expr()->andX(
                    $qb->expr()->eq('s.locomotive_series', "'$loco_type'"), $qb->expr()->neq('s.uuid', "'null'")
                )
            )->orderBy('s.id', 'ASC');
        if ($asArray)
            $found = $qb->getQuery()->getResult(Doctrine\ORM\Query::HYDRATE_ARRAY);
        else
            $found = $qb->getQuery()->getResult();

        return $found;
    }

    public static function FindByNumbers($loco_type, $number, $subnumber)
    {
        global $entityManager;
        return $entityManager->getRepository('Section')->findBy(array('locomotive_series'=>$loco_type,'section_number' => $number, 'section_subnumber' => $subnumber));
    }

    public static function FindByLetter($loco_type, $number, $letter)
    {
        global $entityManager;
        $sections =  $entityManager->getRepository('Section')->findBy(array('locomotive_series'=>$loco_type,'section_number' => $number, 'letter' => $letter));
//        \Doctrine\Common\Util\Debug::dump($sections);
        return $sections;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getSectionNumber()
    {
        return $this->section_number;
    }

    public function setSectionNumber(string $section_number): self
    {
        $this->section_number = $section_number;

        return $this;
    }
 	
    public function getSectionSubnumber(): string
    {
        return $this->section_subnumber;
    }

    public function setSectionSubnumber(string $section_subnumber): self
    {
        $this->section_subnumber = $section_subnumber;

        return $this;
    }

    public function getTypeRepair(): string
    {
        return $this->type_repair;
    }

    public function setTypeRepair(string $type_repair): self
    {
        $this->type_repair = $type_repair;

        return $this;
    }

    public function getDateStartRepair()
    {
        return $this->date_start_repair;
    }

    public function setDateStartRepair($date_start_repair): self
    {
        $this->date_start_repair = $date_start_repair;

        return $this;
    }

    public function setLetter(string $letter)
    {
        $this->letter = $letter;
    }
}
